/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSAffineTransform;

import dstep.applicationservices.coregraphics.CGAffineTransform;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

struct NSAffineTransformStruct
{
	CGFloat m11;
	CGFloat m12;
	CGFloat m21;
	CGFloat m22;
	CGFloat tX;
	CGFloat tY;
}

class NSAffineTransform : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;

	static NSAffineTransform transform ()
	{
		return invokeObjcSelfClass!(NSAffineTransform, "transform");
	}

	Object initWithTransform (NSAffineTransform transform)
	{
		return invokeObjcSelf!(Object, "initWithTransform:", NSAffineTransform)(transform);
	}

	this (NSAffineTransform transform)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithTransform:", NSAffineTransform)(objcObject, transform);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void translateXBy (CGFloat deltaX, CGFloat deltaY)
	{
		return invokeObjcSelf!(void, "translateXBy:yBy:", CGFloat, CGFloat)(deltaX, deltaY);
	}

	void rotateByDegrees (CGFloat angle)
	{
		return invokeObjcSelf!(void, "rotateByDegrees:", CGFloat)(angle);
	}

	void rotateByRadians (CGFloat angle)
	{
		return invokeObjcSelf!(void, "rotateByRadians:", CGFloat)(angle);
	}

	void scaleBy (CGFloat scale)
	{
		return invokeObjcSelf!(void, "scaleBy:", CGFloat)(scale);
	}

	void scaleXBy (CGFloat scaleX, CGFloat scaleY)
	{
		return invokeObjcSelf!(void, "scaleXBy:yBy:", CGFloat, CGFloat)(scaleX, scaleY);
	}

	void invert ()
	{
		return invokeObjcSelf!(void, "invert");
	}

	void appendTransform (NSAffineTransform transform)
	{
		return invokeObjcSelf!(void, "appendTransform:", NSAffineTransform)(transform);
	}

	void prependTransform (NSAffineTransform transform)
	{
		return invokeObjcSelf!(void, "prependTransform:", NSAffineTransform)(transform);
	}

	NSPoint transformPoint (NSPoint aPoint)
	{
		return invokeObjcSelf!(NSPoint, "transformPoint:", NSPoint)(aPoint);
	}

	NSSize transformSize (NSSize aSize)
	{
		return invokeObjcSelf!(NSSize, "transformSize:", NSSize)(aSize);
	}

	NSAffineTransformStruct transformStruct ()
	{
		return invokeObjcSelf!(NSAffineTransformStruct, "transformStruct");
	}

	void setTransformStruct (NSAffineTransformStruct transformStruct)
	{
		return invokeObjcSelf!(void, "setTransformStruct:", NSAffineTransformStruct)(transformStruct);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCoder:", NSCoder)(objcObject, aDecoder);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

