/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSHTTPCookieStorage;

import dstep.AvailabilityMacros;
import dstep.foundation.NSArray;
import dstep.foundation.NSHTTPCookie;
import dstep.foundation.NSHTTPCookieStorageInternal;
import dstep.foundation.NSObject;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSHTTPCookieStorage_bindings;

alias NSUInteger NSHTTPCookieAcceptPolicy;

const NSString NSHTTPCookieManagerAcceptPolicyChangedNotification;
const NSString NSHTTPCookieManagerCookiesChangedNotification;

enum
{
	NSHTTPCookieAcceptPolicyAlways,
	NSHTTPCookieAcceptPolicyNever,
	NSHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain
}

static this ()
{
	NSHTTPCookieManagerAcceptPolicyChangedNotification = new NSString(bindings.NSHTTPCookieManagerAcceptPolicyChangedNotification);
	NSHTTPCookieManagerCookiesChangedNotification = new NSString(bindings.NSHTTPCookieManagerCookiesChangedNotification);
}

class NSHTTPCookieStorage : NSObject
{
	mixin ObjcWrap;

	static NSHTTPCookieStorage sharedHTTPCookieStorage ()
	{
		return invokeObjcSelfClass!(NSHTTPCookieStorage, "sharedHTTPCookieStorage");
	}

	NSArray cookies ()
	{
		return invokeObjcSelf!(NSArray, "cookies");
	}

	void setCookie (NSHTTPCookie cookie)
	{
		return invokeObjcSelf!(void, "setCookie:", NSHTTPCookie)(cookie);
	}

	void deleteCookie (NSHTTPCookie cookie)
	{
		return invokeObjcSelf!(void, "deleteCookie:", NSHTTPCookie)(cookie);
	}

	NSArray cookiesForURL (NSURL URL)
	{
		return invokeObjcSelf!(NSArray, "cookiesForURL:", NSURL)(URL);
	}

	void setCookies (NSArray cookies, NSURL URL, NSURL mainDocumentURL)
	{
		return invokeObjcSelf!(void, "setCookies:forURL:mainDocumentURL:", NSArray, NSURL, NSURL)(cookies, URL, mainDocumentURL);
	}

	uint cookieAcceptPolicy ()
	{
		return invokeObjcSelf!(uint, "cookieAcceptPolicy");
	}

	void setCookieAcceptPolicy (uint cookieAcceptPolicy)
	{
		return invokeObjcSelf!(void, "setCookieAcceptPolicy:", uint)(cookieAcceptPolicy);
	}
}

