/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSKeyedArchiver;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSMutableData;
import dstep.foundation.NSPropertyList;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSKeyedArchiver_bindings;

const NSString NSInvalidArchiveOperationException;
const NSString NSInvalidUnarchiveOperationException;

static this ()
{
	NSInvalidArchiveOperationException = new NSString(bindings.NSInvalidArchiveOperationException);
	NSInvalidUnarchiveOperationException = new NSString(bindings.NSInvalidUnarchiveOperationException);
}

class NSKeyedArchiver : NSCoder
{
	mixin ObjcWrap;

	static NSData archivedDataWithRootObject (Object rootObject)
	{
		return invokeObjcSelfClass!(NSData, "archivedDataWithRootObject:", Object)(rootObject);
	}

	static bool archiveRootObject (Object rootObject, NSString path)
	{
		return invokeObjcSelfClass!(bool, "archiveRootObject:toFile:", Object, NSString)(rootObject, path);
	}

	Object initForWritingWithMutableData (NSMutableData data)
	{
		return invokeObjcSelf!(Object, "initForWritingWithMutableData:", NSMutableData)(data);
	}

	this (NSMutableData data)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initForWritingWithMutableData:", NSMutableData)(objcObject, data);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void setOutputFormat (uint format)
	{
		return invokeObjcSelf!(void, "setOutputFormat:", uint)(format);
	}

	uint outputFormat ()
	{
		return invokeObjcSelf!(uint, "outputFormat");
	}

	void finishEncoding ()
	{
		return invokeObjcSelf!(void, "finishEncoding");
	}

	static void setClassName (NSString codedName, Class cls)
	{
		return invokeObjcSelfClass!(void, "setClassName:forClass:", NSString, Class)(codedName, cls);
	}

	void setClassName (NSString codedName, Class cls)
	{
		return invokeObjcSelf!(void, "setClassName:forClass:", NSString, Class)(codedName, cls);
	}

	static NSString classNameForClass (Class cls)
	{
		return invokeObjcSelfClass!(NSString, "classNameForClass:", Class)(cls);
	}

	NSString classNameForClass (Class cls)
	{
		return invokeObjcSelf!(NSString, "classNameForClass:", Class)(cls);
	}

	void encodeObject (Object objv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeObject:forKey:", Object, NSString)(objv, key);
	}

	void encodeConditionalObject (Object objv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeConditionalObject:forKey:", Object, NSString)(objv, key);
	}

	void encodeBool (bool boolv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeBool:forKey:", bool, NSString)(boolv, key);
	}

	void encodeInt (int intv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeInt:forKey:", int, NSString)(intv, key);
	}

	void encodeInt32 (int intv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeInt32:forKey:", int, NSString)(intv, key);
	}

	void encodeInt64 (long intv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeInt64:forKey:", long, NSString)(intv, key);
	}

	void encodeFloat (float realv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeFloat:forKey:", float, NSString)(realv, key);
	}

	void encodeDouble (double realv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeDouble:forKey:", double, NSString)(realv, key);
	}

	void encodeBytes (char* bytesp, NSUInteger lenv, NSString key)
	{
		return invokeObjcSelf!(void, "encodeBytes:length:forKey:", char*, NSUInteger, NSString)(bytesp, lenv, key);
	}
}

class NSKeyedUnarchiver : NSCoder
{
	mixin ObjcWrap;

	static Object unarchiveObjectWithData (NSData data)
	{
		return invokeObjcSelfClass!(Object, "unarchiveObjectWithData:", NSData)(data);
	}

	static Object unarchiveObjectWithFile (NSString path)
	{
		return invokeObjcSelfClass!(Object, "unarchiveObjectWithFile:", NSString)(path);
	}

	Object initForReadingWithData (NSData data)
	{
		return invokeObjcSelf!(Object, "initForReadingWithData:", NSData)(data);
	}

	this (NSData data)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initForReadingWithData:", NSData)(objcObject, data);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void setDelegate (Object delegate_)
	{
		return invokeObjcSelf!(void, "setDelegate:", Object)(delegate_);
	}

	Object delegate_ ()
	{
		return invokeObjcSelf!(Object, "delegate");
	}

	void finishDecoding ()
	{
		return invokeObjcSelf!(void, "finishDecoding");
	}

	static void setClass (Class cls, NSString codedName)
	{
		return invokeObjcSelfClass!(void, "setClass:forClassName:", Class, NSString)(cls, codedName);
	}

	void setClass (Class cls, NSString codedName)
	{
		return invokeObjcSelf!(void, "setClass:forClassName:", Class, NSString)(cls, codedName);
	}

	static Class classForClassName (NSString codedName)
	{
		return invokeObjcSelfClass!(Class, "classForClassName:", NSString)(codedName);
	}

	Class classForClassName (NSString codedName)
	{
		return invokeObjcSelf!(Class, "classForClassName:", NSString)(codedName);
	}

	bool containsValueForKey (NSString key)
	{
		return invokeObjcSelf!(bool, "containsValueForKey:", NSString)(key);
	}

	Object decodeObjectForKey (NSString key)
	{
		return invokeObjcSelf!(Object, "decodeObjectForKey:", NSString)(key);
	}

	bool decodeBoolForKey (NSString key)
	{
		return invokeObjcSelf!(bool, "decodeBoolForKey:", NSString)(key);
	}

	int decodeIntForKey (NSString key)
	{
		return invokeObjcSelf!(int, "decodeIntForKey:", NSString)(key);
	}

	int decodeInt32ForKey (NSString key)
	{
		return invokeObjcSelf!(int, "decodeInt32ForKey:", NSString)(key);
	}

	long decodeInt64ForKey (NSString key)
	{
		return invokeObjcSelf!(long, "decodeInt64ForKey:", NSString)(key);
	}

	float decodeFloatForKey (NSString key)
	{
		return invokeObjcSelf!(float, "decodeFloatForKey:", NSString)(key);
	}

	double decodeDoubleForKey (NSString key)
	{
		return invokeObjcSelf!(double, "decodeDoubleForKey:", NSString)(key);
	}

	char* decodeBytesForKey (NSString key, NSUInteger* lengthp)
	{
		return invokeObjcSelf!(char*, "decodeBytesForKey:returnedLength:", NSString, NSUInteger*)(key, lengthp);
	}
}

template TNSKeyedArchiverObjectSubstitution ()
{
	Class classForKeyedArchiver ();
	Object replacementObjectForKeyedArchiver (NSKeyedArchiver archiver);
	static NSArray classFallbacksForKeyedArchiver ();
}

template TNSKeyedUnarchiverObjectSubstitution ()
{
	static Class classForKeyedUnarchiver ();
}

template TNSKeyedUnarchiverDelegate ()
{
	Class unarchiver (NSKeyedUnarchiver unarchiver, NSString name, NSArray classNames);
	Object unarchiver (NSKeyedUnarchiver unarchiver, Object object);
	void unarchiver (NSKeyedUnarchiver unarchiver, Object object, Object newObject);
	void unarchiverWillFinish (NSKeyedUnarchiver unarchiver);
	void unarchiverDidFinish (NSKeyedUnarchiver unarchiver);
}

template TNSKeyedArchiverDelegate ()
{
	Object archiver (NSKeyedArchiver archiver, Object object);
	void archiver (NSKeyedArchiver archiver, Object object);
	void archiver (NSKeyedArchiver archiver, Object object, Object newObject);
	void archiverWillFinish (NSKeyedArchiver archiver);
	void archiverDidFinish (NSKeyedArchiver archiver);
}

template TNSGeometryKeyedCoding ()
{
	void encodePoint (NSPoint point, NSString key)
	{
		return invokeObjcSelf!(void, "encodePoint:forKey:", NSPoint, NSString)(point, key);
	}

	void encodeSize (NSSize size, NSString key)
	{
		return invokeObjcSelf!(void, "encodeSize:forKey:", NSSize, NSString)(size, key);
	}

	void encodeRect (NSRect rect, NSString key)
	{
		return invokeObjcSelf!(void, "encodeRect:forKey:", NSRect, NSString)(rect, key);
	}

	NSPoint decodePointForKey (NSString key)
	{
		return invokeObjcSelf!(NSPoint, "decodePointForKey:", NSString)(key);
	}

	NSSize decodeSizeForKey (NSString key)
	{
		return invokeObjcSelf!(NSSize, "decodeSizeForKey:", NSString)(key);
	}

	NSRect decodeRectForKey (NSString key)
	{
		return invokeObjcSelf!(NSRect, "decodeRectForKey:", NSString)(key);
	}
}

