/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSTimeZone;

import dstep.foundation.NSArray;
import dstep.foundation.NSData;
import dstep.foundation.NSDate;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSLocale;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSTimeZone_bindings;

alias NSInteger NSTimeZoneNameStyle;

const NSString NSSystemTimeZoneDidChangeNotification;

enum
{
	NSTimeZoneNameStyleStandard,
	NSTimeZoneNameStyleShortStandard,
	NSTimeZoneNameStyleDaylightSaving,
	NSTimeZoneNameStyleShortDaylightSaving
}

static this ()
{
	NSSystemTimeZoneDidChangeNotification = new NSString(bindings.NSSystemTimeZoneDidChangeNotification);
}

class NSTimeZone : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;
	mixin TNSExtendedTimeZone;
	mixin TNSTimeZoneCreation;

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	NSData data ()
	{
		return invokeObjcSelf!(NSData, "data");
	}

	NSInteger secondsFromGMTForDate (NSDate aDate)
	{
		return invokeObjcSelf!(NSInteger, "secondsFromGMTForDate:", NSDate)(aDate);
	}

	NSString abbreviationForDate (NSDate aDate)
	{
		return invokeObjcSelf!(NSString, "abbreviationForDate:", NSDate)(aDate);
	}

	bool isDaylightSavingTimeForDate (NSDate aDate)
	{
		return invokeObjcSelf!(bool, "isDaylightSavingTimeForDate:", NSDate)(aDate);
	}

	double daylightSavingTimeOffsetForDate (NSDate aDate)
	{
		return invokeObjcSelf!(double, "daylightSavingTimeOffsetForDate:", NSDate)(aDate);
	}

	NSDate nextDaylightSavingTimeTransitionAfterDate (NSDate aDate)
	{
		return invokeObjcSelf!(NSDate, "nextDaylightSavingTimeTransitionAfterDate:", NSDate)(aDate);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCoder:", NSCoder)(objcObject, aDecoder);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

template TNSExtendedTimeZone ()
{
	static NSTimeZone systemTimeZone ()
	{
		return invokeObjcSelfClass!(NSTimeZone, "systemTimeZone");
	}

	static void resetSystemTimeZone ()
	{
		return invokeObjcSelfClass!(void, "resetSystemTimeZone");
	}

	static NSTimeZone defaultTimeZone ()
	{
		return invokeObjcSelfClass!(NSTimeZone, "defaultTimeZone");
	}

	static void setDefaultTimeZone (NSTimeZone aTimeZone)
	{
		return invokeObjcSelfClass!(void, "setDefaultTimeZone:", NSTimeZone)(aTimeZone);
	}

	static NSTimeZone localTimeZone ()
	{
		return invokeObjcSelfClass!(NSTimeZone, "localTimeZone");
	}

	static NSArray knownTimeZoneNames ()
	{
		return invokeObjcSelfClass!(NSArray, "knownTimeZoneNames");
	}

	static NSDictionary abbreviationDictionary ()
	{
		return invokeObjcSelfClass!(NSDictionary, "abbreviationDictionary");
	}

	NSInteger secondsFromGMT ()
	{
		return invokeObjcSelf!(NSInteger, "secondsFromGMT");
	}

	NSString abbreviation ()
	{
		return invokeObjcSelf!(NSString, "abbreviation");
	}

	bool isDaylightSavingTime ()
	{
		return invokeObjcSelf!(bool, "isDaylightSavingTime");
	}

	double daylightSavingTimeOffset ()
	{
		return invokeObjcSelf!(double, "daylightSavingTimeOffset");
	}

	NSDate nextDaylightSavingTimeTransition ()
	{
		return invokeObjcSelf!(NSDate, "nextDaylightSavingTimeTransition");
	}

	NSString description ()
	{
		return invokeObjcSelf!(NSString, "description");
	}

	bool isEqualToTimeZone (NSTimeZone aTimeZone)
	{
		return invokeObjcSelf!(bool, "isEqualToTimeZone:", NSTimeZone)(aTimeZone);
	}

	NSString localizedName (int style, NSLocale locale)
	{
		return invokeObjcSelf!(NSString, "localizedName:locale:", int, NSLocale)(style, locale);
	}
}

template TNSTimeZoneCreation ()
{
	static Object timeZoneWithName (NSString tzName)
	{
		return invokeObjcSelfClass!(Object, "timeZoneWithName:", NSString)(tzName);
	}

	static Object timeZoneWithName (NSString tzName, NSData aData)
	{
		return invokeObjcSelfClass!(Object, "timeZoneWithName:data:", NSString, NSData)(tzName, aData);
	}

	Object initWithName (NSString tzName)
	{
		return invokeObjcSelf!(Object, "initWithName:", NSString)(tzName);
	}

	this (NSString tzName)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithName:", NSString)(objcObject, tzName);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithName (NSString tzName, NSData aData)
	{
		return invokeObjcSelf!(Object, "initWithName:data:", NSString, NSData)(tzName, aData);
	}

	this (NSString tzName, NSData aData)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithName:data:", NSString, NSData)(objcObject, tzName, aData);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	static Object timeZoneForSecondsFromGMT (NSInteger seconds)
	{
		return invokeObjcSelfClass!(Object, "timeZoneForSecondsFromGMT:", NSInteger)(seconds);
	}

	static Object timeZoneWithAbbreviation (NSString abbreviation)
	{
		return invokeObjcSelfClass!(Object, "timeZoneWithAbbreviation:", NSString)(abbreviation);
	}
}

