/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSXMLElement;

import dstep.foundation.NSDictionary;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSMutableArray;
import dstep.foundation.NSXMLNode;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

class NSXMLElement : NSXMLNode
{
	mixin ObjcWrap;

	Object initWithName (NSString name)
	{
		return invokeObjcSelf!(Object, "initWithName:", NSString)(name);
	}

	this (NSString name)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithName:", NSString)(objcObject, name);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithName (NSString name, NSString URI)
	{
		return invokeObjcSelf!(Object, "initWithName:URI:", NSString, NSString)(name, URI);
	}

	this (NSString name, NSString URI)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithName:URI:", NSString, NSString)(objcObject, name, URI);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithName (NSString name, NSString string)
	{
		return invokeObjcSelf!(Object, "initWithName:stringValue:", NSString, NSString)(name, string);
	}

	this (NSString name, NSString string)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithName:stringValue:", NSString, NSString)(objcObject, name, string);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithXMLString (NSString string, NSError** error)
	{
		return invokeObjcSelf!(Object, "initWithXMLString:error:", NSString, NSError**)(string, error);
	}

	this (NSString string, NSError** error)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithXMLString:error:", NSString, NSError**)(objcObject, string, error);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSArray elementsForName (NSString name)
	{
		return invokeObjcSelf!(NSArray, "elementsForName:", NSString)(name);
	}

	NSArray elementsForLocalName (NSString localName, NSString URI)
	{
		return invokeObjcSelf!(NSArray, "elementsForLocalName:URI:", NSString, NSString)(localName, URI);
	}

	void addAttribute (NSXMLNode attribute)
	{
		return invokeObjcSelf!(void, "addAttribute:", NSXMLNode)(attribute);
	}

	void removeAttributeForName (NSString name)
	{
		return invokeObjcSelf!(void, "removeAttributeForName:", NSString)(name);
	}

	void setAttributes (NSArray attributes)
	{
		return invokeObjcSelf!(void, "setAttributes:", NSArray)(attributes);
	}

	void setAttributesAsDictionary (NSDictionary attributes)
	{
		return invokeObjcSelf!(void, "setAttributesAsDictionary:", NSDictionary)(attributes);
	}

	NSArray attributes ()
	{
		return invokeObjcSelf!(NSArray, "attributes");
	}

	NSXMLNode attributeForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "attributeForName:", NSString)(name);
	}

	NSXMLNode attributeForLocalName (NSString localName, NSString URI)
	{
		return invokeObjcSelf!(NSXMLNode, "attributeForLocalName:URI:", NSString, NSString)(localName, URI);
	}

	void addNamespace (NSXMLNode aNamespace)
	{
		return invokeObjcSelf!(void, "addNamespace:", NSXMLNode)(aNamespace);
	}

	void removeNamespaceForPrefix (NSString name)
	{
		return invokeObjcSelf!(void, "removeNamespaceForPrefix:", NSString)(name);
	}

	void setNamespaces (NSArray namespaces)
	{
		return invokeObjcSelf!(void, "setNamespaces:", NSArray)(namespaces);
	}

	NSArray namespaces ()
	{
		return invokeObjcSelf!(NSArray, "namespaces");
	}

	NSXMLNode namespaceForPrefix (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "namespaceForPrefix:", NSString)(name);
	}

	NSXMLNode resolveNamespaceForName (NSString name)
	{
		return invokeObjcSelf!(NSXMLNode, "resolveNamespaceForName:", NSString)(name);
	}

	NSString resolvePrefixForNamespaceURI (NSString namespaceURI)
	{
		return invokeObjcSelf!(NSString, "resolvePrefixForNamespaceURI:", NSString)(namespaceURI);
	}

	void insertChild (NSXMLNode child, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChild:atIndex:", NSXMLNode, NSUInteger)(child, index);
	}

	void insertChildren (NSArray children, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertChildren:atIndex:", NSArray, NSUInteger)(children, index);
	}

	void removeChildAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeChildAtIndex:", NSUInteger)(index);
	}

	void setChildren (NSArray children)
	{
		return invokeObjcSelf!(void, "setChildren:", NSArray)(children);
	}

	void addChild (NSXMLNode child)
	{
		return invokeObjcSelf!(void, "addChild:", NSXMLNode)(child);
	}

	void replaceChildAtIndex (NSUInteger index, NSXMLNode node)
	{
		return invokeObjcSelf!(void, "replaceChildAtIndex:withNode:", NSUInteger, NSXMLNode)(index, node);
	}

	void normalizeAdjacentTextNodesPreservingCDATA (bool preserve)
	{
		return invokeObjcSelf!(void, "normalizeAdjacentTextNodesPreservingCDATA:", bool)(preserve);
	}
}

